﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Editors;
using System.Linq;

namespace XamComboEditorEx
{
    public class XamComboEditorEx : XamComboEditor
    {
        public XamComboEditorEx()
            : base()
        {
            this.SelectionChanged += combo_SelectionChanged;
        }

        #region SelectedValuePath
        public string SelectedValuePath
        {
            get { return (string)GetValue(SelectedValuePathProperty); }
            set { SetValue(SelectedValuePathProperty, value); }
        }

        public static readonly DependencyProperty SelectedValuePathProperty =
            DependencyProperty.Register("SelectedValuePath", typeof(string), typeof(XamComboEditorEx),
            new PropertyMetadata(new PropertyChangedCallback(SelectedValuePath_Changed)));

        private static void SelectedValuePath_Changed(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var combo = d as XamComboEditorEx;
            if (combo != null)
            {
                combo.ResetSelectedValue();
            }
        }
        #endregion //SelectedValuePath

        #region SelectedValue
        public object SelectedValue
        {
            get { return (object)GetValue(SelectedValueProperty); }
            set
            {
                if (!string.IsNullOrEmpty(this.SelectedValuePath))
                {
                    var q = (from item in Items
                             where item.Data.GetType().GetProperty(this.SelectedValuePath).GetValue(item.Data, null).Equals(value)
                             select item).FirstOrDefault();

                    if (q == null)
                        this.SelectedItem = null;
                    else
                        this.SelectedItem = q.Data;
                }
            }
        }

        public static readonly DependencyProperty SelectedValueProperty =
            DependencyProperty.Register("SelectedValue", typeof(object), typeof(XamComboEditorEx),
            new PropertyMetadata(new PropertyChangedCallback(SelectedValue_Changed)));

        private static void SelectedValue_Changed(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            var combo = d as XamComboEditorEx;
            if (combo != null)
            {
                combo.SelectedValue = e.NewValue;
            }
        }
        #endregion //SelectedValue

        static void combo_SelectionChanged(object sender, EventArgs e)
        {
            var combo = sender as XamComboEditorEx;
            if (combo != null)
                combo.ResetSelectedValue();
        }

        private void ResetSelectedValue()
        {
            if (this.SelectedItem != null &&
                !string.IsNullOrEmpty(this.SelectedValuePath) &&
                this.SelectedItem.GetType().GetProperty(this.SelectedValuePath) != null)
            {
                var propInfo = this.SelectedItem.GetType().GetProperty(this.SelectedValuePath);
                this.SetSelectedValueSilently(propInfo.GetValue(this.SelectedItem, null));
            }
            else
                this.SetSelectedValueSilently(null);
        }

        private void SetSelectedValueSilently(object value)
        {
            this.SetValue(XamComboEditorEx.SelectedValueProperty, value);
        }
    }
}
